@@ -3,7 +3,7 @@ import traceback
 import yaml
 from comptests import comptest, run_module_tests
 
-from duckietown_challenges.challenge import (
+from duckietown_challenges import (
     ChallengeDescription,
     ServiceDefinition,
     EvaluationParameters,
@@ -100,12 +100,12 @@ transitions:
 
 @comptest
 def read_challenge_1():
-    d = yaml.load(data)
+    d = yaml.load(data, Loader=yaml.Loader)
 
     c0: ChallengeDescription = ChallengeDescription.from_yaml(d)
 
     y = c0.as_yaml()
-    d2 = yaml.load(y)
+    d2 = yaml.load(y, Loader=yaml.Loader)
     print(y)
     c: ChallengeDescription = ChallengeDescription.from_yaml(d2)
 
